==> snaptoolbelt-0.14.0-py2-none-any.whl: snaptoolbelt/low_level.py <==
# -*- coding: utf-8 -*-
"""SNAPtoolbelt Low Level Commands"""

import logging

from arghandler import ArgumentHandler

import snapstraction.gus
from snapstraction.util import parse_value_for_snap
import snapstraction.low_level

from snaptoolbelt.node import IncompleteResultError

log = logging.getLogger(__name__)


def rpc(parser, context, args):
    '''Functions for callback/fire-and-forget RPC commands'''
    handler = ArgumentHandler(usage="snap [global options] rpc (call|send) [-h]",
                              description="Commands for communicating with a network via SNAP RPCs.")
    handler.set_subcommands({
        'call': rpc_call,
        'send': rpc_send,
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def rpc_call(parser, context, args):
    """Functions for sending a callback unicast/mcast/dmcast RPC"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc call (u|unicast|m|multicast|d|dm|directed-multicast) [-h]",
        description="Commands for communicating with a network via callback SNAP RPCs.",
    )

    handler.set_subcommands({
        'u': callback_ucast_rpc,
        'unicast': callback_ucast_rpc,
        'm': callback_mcast_rpc,
        'multicast': callback_mcast_rpc,
        'd': callback_dmcast_rpc,
        'dm': callback_dmcast_rpc,
        'directed-multicast': callback_dmcast_rpc
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def callback_ucast_rpc(parser, context, args):
    """Issue a callback unicast RPC"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc call unicast <target> <func> [<args>...] [-h] [-c <callback-name>]",
        description="Send out a callback unicast RPC to a target node.",
    )

    handler.add_argument('target',
                         help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-c',
                         '--callback-name',
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    log.info(
        "Callback_rpc function %s, target %s expecting %r with args: %r",
        args.func, args.target, args.callback_name, rest)

    with context.gus() as gus:
        try:
            result = gus.run_sync(context.timeout, snapstraction.low_level.callback_ucast_rpc,
                                  args.target, args.func, *rest,
                                  callback_name=args.callback_name)

        except snapstraction.gus.NoResponseError:
            no_result = {args.target: None}
            context.output(no_result)
            raise

        clean_result = {args.target: result['response']}
        context.output(clean_result)


def callback_mcast_rpc(parser, context, args):
    """Issue a callback_mcast to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc call multicast <targets> <func> [<args>...] [-h] [-g <group>] [-t <ttl>] [-c <callback-name>]",
        description="Send out a callback multicast RPC to the target nodes.",
    )

    handler.add_argument('targets',
                         help="comma-delineated string of 6-char node addresses, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('-c',
                         '--callback-name',
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    args.targets = [addr.strip() for addr in args.targets.split(',')]

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info(
        "Callback_mcast targets %s, group %r, ttl %r, function %s, expecting %r with args: %r",
        args.targets, args.group, args.ttl, args.func, args.callback_name,
        rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.callback_mcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group, ttl=args.ttl,
                              callback_name=args.callback_name)

        clean_result = {}
        for node, response in result['response'].iteritems():
            clean_result[node] = response

        context.output(clean_result)

        if None in result['response'].itervalues():
            raise IncompleteResultError


def callback_dmcast_rpc(parser, context, args):
    """Issue a callback_dmcast to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc call directed-multicast <targets> <func> [<args>...] [-h] [-g <group>] [-t <ttl>] [-d <delay>] [-c <callback-name>]",
        description="Send out a callback directed multicast RPC to the target nodes.",
    )

    handler.add_argument('targets',
                         help="comma-delineated string of 6-char node addresses, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('-d',
                         '--delay',
                         type=int,
                         help="response delay (in ms)",
                         default=None)
    handler.add_argument('-c',
                         '--callback-name',
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    args.targets = [addr.strip() for addr in args.targets.split(',')]

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info(
        "Callback_dmcast targets %s, group %r, ttl %r, delay %r, function %s, expecting %r with args: %r",
        args.targets, args.group, args.ttl, args.delay, args.func,
        args.callback_name, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.callback_dmcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group,
                              ttl=args.ttl,
                              dmcast_delay=args.delay,
                              callback_name=args.callback_name)

        clean_result = {}
        for node, response in result['response'].responses.iteritems():
            clean_result[node] = response

        context.output(clean_result)

        if result['response'].failed:
            raise IncompleteResultError


def rpc_send(parser, context, args):
    """Functions for sending a fire-and-forget unicast/mcast/dmcast RPC"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc send (u|unicast|m|multicast|d|dm|directed-multicast) [-h]",
        description="Commands for communicating with a network via fire-and-forget SNAP RPCs.",
    )

    handler.set_subcommands({
        'u': ucast_rpc,
        'unicast': ucast_rpc,
        'm': mcast_rpc,
        'multicast': mcast_rpc,
        'd': dmcast_rpc,
        'dm': dmcast_rpc,
        'directed-multicast': dmcast_rpc,
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def ucast_rpc(parser, context, args):
    """Issue a fire-and-forget RPC to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc send unicast <target> <func> [<args>...] [-h]",
        description="Send out a fire-and-forget unicast RPC to a target node.",
    )

    handler.add_argument('target',
                         help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    log.info("RPC function %s, target %s, args %r", args.func,
             args.target, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.ucast_rpc,
                              args.target, args.func, *rest)
        context.output(result)


def mcast_rpc(parser, context, args):
    """Issue a fire-and-forget multicast RPC to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc send multicast <func> [<args>...] [-h] [-g <group>] [-t <ttl>]",
        description="Send out a fire-and-forget multicast RPC.",
    )

    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info("MCAST RPC group %r, ttl %r, func %s, args %r", args.group,
             args.ttl, args.func, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.mcast_rpc,
                              args.func, *rest, group=args.group, ttl=args.ttl)
        context.output(result)


def dmcast_rpc(parser, context, args):
    """Issue a fire-and-forget directed multicast RPC to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc send directed-multicast <targets> <func> [<args>...] [-h] [-g <group>] [-t <ttl>] [-d <delay>]",
        description="Send out a fire-and-forget directed multicast RPC to the target nodes.",
    )

    handler.add_argument('targets',
                         help="comma-delineated string of 6-char node addresses, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('-d',
                         '--delay',
                         type=int,
                         help="delay",
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    # Split targets
    args.targets = [s.strip() for s in args.targets.split(',')]

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info(
        "DMCAST RPC targets %r, group %r, ttl %r, delay %r, func %s, args %r",
        args.targets, args.group, args.ttl, args.delay, args.func, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.dmcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group, ttl=args.ttl, delay_factor=args.delay)
        context.output(result)


def data(parser, context, args):
    """functions for callback/fire-and-forget data mode packets"""
    handler = ArgumentHandler(
        usage="snap [global options] data (send) [-h]",
        description="Commands for communicating with a network via SNAP data mode packets.",
    )

    handler.set_subcommands({
        'send': data_send,
        # Not implemented (yet?)
        # 'call': data_call
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def data_send(parser, context, args):
    """Functions for sending a fire-and-forget unicast/mcast/dmcast data mode packet"""
    handler = ArgumentHandler(
        usage="snap [global options] data send (u|unicast|m|multicast) [-h]",
        description="Commands for communicating with a network via fire-and-forget SNAP data mode packets.",
    )

    handler.set_subcommands({
        'u': ucast_data,
        'unicast': ucast_data,
        'm': mcast_data,
        'multicast': mcast_data,
        # Not implemented (yet?)
        # 'd': dmcast_data,
        # 'dm': dmcast_data,
        # 'directed-multicast': dmcast_data
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def ucast_data(parser, context, args):
    """Issue a fire-and-forget data mode packet to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] data send unicast <target> <data> [-h]",
        description="Send out a fire-and-forget unicast data mode packet to a target node.",
    )

    handler.add_argument('target',
                         help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('data',
                         help="data to send")
    handler.add_argument('--base64',
                         action="store_true",
                         help="data is base64-encoded")

    args = handler.parse_args(args)

    log.info("Unicast data -> target %s, payload %r", args.target, args.data)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.ucast_data,
                              args.target,
                              args.data,
                              base64=args.base64)
        context.output(result)


def mcast_data(parser, context, args):
    """Issue a fire-and-forget multicast data mode packet to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] data send multicast <data> [-h] [-g <group>] [-t <ttl>] [--base64]",
        description="Send out a fire-and-forget multicast data mode packet.",
    )

    handler.add_argument('data',
                         help="data to send")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('--base64',
                         action="store_true",
                         help="data is base64-encoded")

    args = handler.parse_args(args)

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info("Multicast data -> group %r, ttl %r, payload %r", args.group,
             args.ttl, args.data)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.mcast_data,
                              args.data,
                              base64=args.base64,
                              group=args.group,
                              ttl=args.ttl)
        context.output(result)

==> snaptoolbelt-2.0.0-py2-none-any.whl: snaptoolbelt/low_level.py <==
# -*- coding: utf-8 -*-
"""SNAPtoolbelt Low Level Commands"""

import logging

from arghandler import ArgumentHandler

import snapstraction.gus
from snapstraction.util import parse_value_for_snap
import snapstraction.low_level

from .alias import Aliases

from snaptoolbelt.node import IncompleteResultError

log = logging.getLogger(__name__)


def rpc(parser, context, args):
    '''Functions for callback/fire-and-forget RPC commands'''
    handler = ArgumentHandler(usage="snap [global options] rpc (call|send) [-h]",
                              description="Commands for communicating with a network via SNAP RPCs.")
    handler.set_subcommands({
        'call': rpc_call,
        'send': rpc_send,
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def rpc_call(parser, context, args):
    """Functions for sending a callback unicast/mcast/dmcast RPC"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc call (u|unicast|m|multicast|d|dm|directed-multicast) [-h]",
        description="Commands for communicating with a network via callback SNAP RPCs.",
    )

    handler.set_subcommands({
        'u': callback_ucast_rpc,
        'unicast': callback_ucast_rpc,
        'm': callback_mcast_rpc,
        'multicast': callback_mcast_rpc,
        'd': callback_dmcast_rpc,
        'dm': callback_dmcast_rpc,
        'directed-multicast': callback_dmcast_rpc
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def callback_ucast_rpc(parser, context, args):
    """Issue a callback unicast RPC"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc call unicast <target> <func> [<args>...] [-h] [-c <callback-name>]",
        description="Send out a callback unicast RPC to a target node.",
    )

    handler.add_argument('target',
                         help="6-char node address, or 'bridge' for the bridge node, or a defined alias")
    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-c',
                         '--callback-name',
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    # Translate aliases
    with Aliases() as aliases:
        args.target = aliases.translate(args.target)

    log.info(
        "Callback_rpc function %s, target %s expecting %r with args: %r",
        args.func, args.target, args.callback_name, rest)

    with context.gus() as gus:
        try:
            result = gus.run_sync(context.timeout, snapstraction.low_level.callback_ucast_rpc,
                                  args.target, args.func, *rest,
                                  callback_name=args.callback_name)

        except snapstraction.gus.NoResponseError:
            no_result = {args.target: None}
            context.output(no_result)
            raise

        clean_result = {args.target: result['response']}
        context.output(clean_result)


def callback_mcast_rpc(parser, context, args):
    """Issue a callback_mcast to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc call multicast <targets> <func> [<args>...] [-h] [-g <group>] [-t <ttl>] [-c <callback-name>]",
        description="Send out a callback multicast RPC to the target nodes.",
    )

    handler.add_argument('targets',
                         help="comma-delineated string of 6-char node addresses, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('-c',
                         '--callback-name',
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    # Translate aliases
    with Aliases() as aliases:
        args.targets = [aliases.translate(addr.strip()) for addr in args.targets.split(',')]

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info(
        "Callback_mcast targets %s, group %r, ttl %r, function %s, expecting %r with args: %r",
        args.targets, args.group, args.ttl, args.func, args.callback_name,
        rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.callback_mcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group, ttl=args.ttl,
                              callback_name=args.callback_name)

        clean_result = {}
        for node, response in result['response'].iteritems():
            clean_result[node] = response

        context.output(clean_result)

        if None in result['response'].itervalues():
            raise IncompleteResultError


def callback_dmcast_rpc(parser, context, args):
    """Issue a callback_dmcast to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc call directed-multicast <targets> <func> [<args>...] [-h] [-g <group>] [-t <ttl>] [-d <delay>] [-c <callback-name>]",
        description="Send out a callback directed multicast RPC to the target nodes.",
    )

    handler.add_argument('targets',
                         help="comma-delineated string of 6-char node addresses, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('-d',
                         '--delay',
                         type=int,
                         help="response delay (in ms)",
                         default=None)
    handler.add_argument('-c',
                         '--callback-name',
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = tuple([parse_value_for_snap(x) for x in rest])

    # Translate aliases
    with Aliases() as aliases:
        args.targets = [aliases.translate(addr.strip()) for addr in args.targets.split(',')]

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info(
        "Callback_dmcast targets %s, group %r, ttl %r, delay %r, function %s, expecting %r with args: %r",
        args.targets, args.group, args.ttl, args.delay, args.func,
        args.callback_name, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.callback_dmcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group,
                              ttl=args.ttl,
                              dmcast_delay=args.delay,
                              callback_name=args.callback_name)

        clean_result = {}
        for node, response in result['response'].responses.iteritems():
            clean_result[node] = response

        context.output(clean_result)

        if result['response'].failed:
            raise IncompleteResultError


def rpc_send(parser, context, args):
    """Functions for sending a fire-and-forget unicast/mcast/dmcast RPC"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc send (u|unicast|m|multicast|d|dm|directed-multicast) [-h]",
        description="Commands for communicating with a network via fire-and-forget SNAP RPCs.",
    )

    handler.set_subcommands({
        'u': ucast_rpc,
        'unicast': ucast_rpc,
        'm': mcast_rpc,
        'multicast': mcast_rpc,
        'd': dmcast_rpc,
        'dm': dmcast_rpc,
        'directed-multicast': dmcast_rpc,
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def ucast_rpc(parser, context, args):
    """Issue a fire-and-forget RPC to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc send unicast <target> <func> [<args>...] [-h]",
        description="Send out a fire-and-forget unicast RPC to a target node.",
    )

    handler.add_argument('target',
                         help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    # Translate aliases
    with Aliases() as aliases:
        args.target = aliases.translate(args.target)

    log.info("RPC function %s, target %s, args %r", args.func,
             args.target, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.ucast_rpc,
                              args.target, args.func, *rest)
        context.output(result)


def mcast_rpc(parser, context, args):
    """Issue a fire-and-forget multicast RPC to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc send multicast <func> [<args>...] [-h] [-g <group>] [-t <ttl>]",
        description="Send out a fire-and-forget multicast RPC.",
    )

    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info("MCAST RPC group %r, ttl %r, func %s, args %r", args.group,
             args.ttl, args.func, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.mcast_rpc,
                              args.func, *rest, group=args.group, ttl=args.ttl)
        context.output(result)


def dmcast_rpc(parser, context, args):
    """Issue a fire-and-forget directed multicast RPC to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] rpc send directed-multicast <targets> <func> [<args>...] [-h] [-g <group>] [-t <ttl>] [-d <delay>]",
        description="Send out a fire-and-forget directed multicast RPC to the target nodes.",
    )

    handler.add_argument('targets',
                         help="comma-delineated string of 6-char node addresses, or 'bridge' for the bridge node")
    handler.add_argument('func',
                         help="function name")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('-d',
                         '--delay',
                         type=int,
                         help="delay",
                         default=None)

    args, rest = handler.parse_known_args(args)
    rest = [parse_value_for_snap(x) for x in rest]

    # Split targets
    with Aliases() as aliases:
        args.targets = [aliases.translate(s.strip()) for s in args.targets.split(',')]

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info(
        "DMCAST RPC targets %r, group %r, ttl %r, delay %r, func %s, args %r",
        args.targets, args.group, args.ttl, args.delay, args.func, rest)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout, snapstraction.low_level.dmcast_rpc,
                              args.targets, args.func, *rest,
                              group=args.group, ttl=args.ttl, delay_factor=args.delay)
        context.output(result)


def data(parser, context, args):
    """functions for callback/fire-and-forget data mode packets"""
    handler = ArgumentHandler(
        usage="snap [global options] data (send) [-h]",
        description="Commands for communicating with a network via SNAP data mode packets.",
    )

    handler.set_subcommands({
        'send': data_send,
        # Not implemented (yet?)
        # 'call': data_call
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def data_send(parser, context, args):
    """Functions for sending a fire-and-forget unicast/mcast/dmcast data mode packet"""
    handler = ArgumentHandler(
        usage="snap [global options] data send (u|unicast|m|multicast) [-h]",
        description="Commands for communicating with a network via fire-and-forget SNAP data mode packets.",
    )

    handler.set_subcommands({
        'u': ucast_data,
        'unicast': ucast_data,
        'm': mcast_data,
        'multicast': mcast_data,
        # Not implemented (yet?)
        # 'd': dmcast_data,
        # 'dm': dmcast_data,
        # 'directed-multicast': dmcast_data
    })

    # The context_fxn passes the global context down to the
    # subcommands so they can get to things like the profile's
    # crypto settings
    handler.run(args, context_fxn=lambda x: context)


def ucast_data(parser, context, args):
    """Issue a fire-and-forget data mode packet to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] data send unicast <target> <data> [-h]",
        description="Send out a fire-and-forget unicast data mode packet to a target node.",
    )

    handler.add_argument('target',
                         help="6-char node address, or 'bridge' for the bridge node")
    handler.add_argument('data',
                         help="data to send")
    handler.add_argument('--base64',
                         action="store_true",
                         help="data is base64-encoded")

    args = handler.parse_args(args)

    # Translate aliases
    with Aliases() as aliases:
        args.target = aliases.translate(args.target)

    log.info("Unicast data -> target %s, payload %r", args.target, args.data)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.ucast_data,
                              args.target,
                              args.data,
                              base64=args.base64)
        context.output(result)


def mcast_data(parser, context, args):
    """Issue a fire-and-forget multicast data mode packet to the node at the given address"""
    handler = ArgumentHandler(
        usage="snap [global options] data send multicast <data> [-h] [-g <group>] [-t <ttl>] [--base64]",
        description="Send out a fire-and-forget multicast data mode packet.",
    )

    handler.add_argument('data',
                         help="data to send")
    handler.add_argument('-g',
                         '--group',
                         help="multicast group",
                         default=None)
    handler.add_argument('-t',
                         '--ttl',
                         type=int,
                         help="multicast ttl",
                         default=None)
    handler.add_argument('--base64',
                         action="store_true",
                         help="data is base64-encoded")

    args = handler.parse_args(args)

    args.group = parse_value_for_snap(args.group) if args.group else None

    log.info("Multicast data -> group %r, ttl %r, payload %r", args.group,
             args.ttl, args.data)

    with context.gus() as gus:
        result = gus.run_sync(context.timeout,
                              snapstraction.low_level.mcast_data,
                              args.data,
                              base64=args.base64,
                              group=args.group,
                              ttl=args.ttl)
        context.output(result)
